package pl.webnet.JaTrac;

import java.util.*;

/**
 * Level handles one level of object (primitive) hierarchy
 * @author: Miloslaw Smyk (thorgal@amiga.com.pl)
 * @version: $Id: Level.java,v 1.3 2001/06/03 01:42:38 thorgal Exp $
 */


class Level extends TreeNode implements XmlSettable
{
	static final public int CSG_OR = 0;
	static final public int CSG_AND = 1;
	static final public int CSG_EXOR = 2;
	static final public int CSG_SUB = 3;

//	String name;
	int csgMode = CSG_OR;
	Vector children;
	Vector mappings;
	Vector lights;
	Vector affectingMappings;
	//BoundingVolume bound;
	Surface averageSurface;

	
	public Level()
	{
		children = new Vector();
		lights = new Vector();
		mappings = new Vector();

		csgMode = CSG_OR;
	}

	public String[] getParamArray()
	{
		final String params[] = {
			"name", "name", "",
			"csgmode", "", "setCSGMode"
		};
		
		return params;
	}

	public void setCSGMode(String csg)
	{
		final String csgVals[] = {"OR", "AND", "EXOR", "SUB"};
		final int csgModes[] = {CSG_OR, CSG_AND, CSG_EXOR, CSG_SUB};

		for(int i = 0; i < csgVals.length; i++)
			if(csg.equals(csgVals[i]))
			{
				csgMode = csgModes[i];
				return;
			}

		System.out.println("setCSGMode: unknown CSG mode passed");
	}

	public void add(Mapping mapping)
	{
		mapping.setParent(this);
		mappings.add(mapping);
	}

	public void add(Light light)
	{
		light.setParent(this);
		lights.add(light);
		Scene.add(light);
	}

	public void add(Primitive prim)
	{
		prim.setParent(this);
		children.add(prim);
	}

	public void add(Level level)
	{
		level.setParent(this);
		children.add(level);
	}

	public void add(TreeNode tn)
	{
		if(tn instanceof Level)
			add((Level)tn);

		if(tn instanceof Mapping)
			add((Mapping)tn);

		if(tn instanceof Light)
			add((Light)tn);

		if(tn instanceof Primitive)
			add((Primitive)tn);
	}

	public String[] getAttachArray()
	{
		String params[] = {
			"Camera", "setCamera", "",
			".TreeNode", "add", ""
		};

		return params;
	}

	public Object initialize() { return this; }

}
