package pl.webnet.JaTrac;

final class SpectrumRGB extends Spectrum implements XmlSettable, Indexable
{
	protected double r, g, b;

	public SpectrumRGB()
	{
		r = g = b = 0.0;
	}

	public SpectrumRGB(double r, double g, double b)
	{
		this.setRGB(r, g, b);
	}

	public SpectrumRGB(int r, int g, int b)
	{
		this.setRGB(r, g, b);
	}

	public SpectrumRGB(int[] rgb)
	{
		this.setRGB(rgb[0], rgb[1], rgb[2]);
	}

	public Object clone() throws CloneNotSupportedException
	{
		Object clone = super.clone();

		((SpectrumRGB)clone).r = r;
		((SpectrumRGB)clone).g = g;
		((SpectrumRGB)clone).b = b;

		return clone;
	}

	public String[] getParamArray()
	{
		final String params[] = {
			"rgb", "", "setRGB",
			"name", "", "setName"
		};

		return params;
	}

	public String[] getAttachArray() { return null; }
	public Object initialize() { return this; }

	public void setRGB(String rgb)
	{
		double col[] = ParamHelper.extract3Doubles(rgb);

		r = col[0];
		g = col[1];
		b = col[2];
	}
	
	public String getName() { return name; }
	public void setName(String s)
	{
		name = s;
		index.add(this);
	}

	public Spectrum create(Spectrum spec)
	{
		if(spec instanceof SpectrumRGB)
		{
			SpectrumRGB o = (SpectrumRGB)spec;
			Spectrum s = new SpectrumRGB();
			s.setRGB(o.r, o.g, o.b);
		
			return s;
		}
		else
			return spec.create(spec);
	}

	public Spectrum create(double r, double g, double b)
	{
		Spectrum s = new SpectrumRGB();
		s.setRGB(r, g, b);

		return s;
	}

	public Spectrum create(int r, int g, int b)
	{
		Spectrum s = new SpectrumRGB();
		s.setRGB(r, g, b);

		return s;
	}

	public Spectrum create(int[] rgb)
	{
		Spectrum s = new SpectrumRGB();
		s.setRGB(rgb[0], rgb[1], rgb[2]);

		return s;
	}

	public void clear()
	{
		r = g = b = 0.0;
	}
	
	public void add(Spectrum spec)
	{
		double[] rgb;
		rgb = spec.getRGB(null);
		
		r += rgb[0];
		g += rgb[1];
		b += rgb[2];
	}

	public void average(Spectrum spec)
	{
		double[] rgb;
		rgb = spec.getRGB(null);
		
		r = (r + rgb[0]) / 2;
		g = (g + rgb[1]) / 2;
		b = (b + rgb[2]) / 2;
	}

	public void MAC(double factor, Spectrum spec)
	{
		double[] rgb;
		rgb = spec.getRGB(null);
		
		r += factor * rgb[0];
		g += factor * rgb[1];
		b += factor * rgb[2];
	}

	public void interact(Spectrum spec)
	{
		double[] rgb;
		rgb = spec.getRGB(null);
		
		r *= rgb[0];
		g *= rgb[1];
		b *= rgb[2];
	}

	public void interact(double factor, Spectrum spec)
	{
		double[] rgb;
		rgb = spec.getRGB(null);
		
		r *= rgb[0] * factor;
		g *= rgb[1] * factor;
		b *= rgb[2] * factor;
	}

	public Spectrum scale(double factor)
	{
		r *= factor;
		g *= factor;
		b *= factor;

		return this;
	}

	public void darken(double factor)
	{
		r -= factor;
		g -= factor;
		b -= factor;

		r = Math.max(r, 0.0);
		g = Math.max(g, 0.0);
		b = Math.max(b, 0.0);
	}

	public void brighten(double factor)
	{
		r += factor;
		g += factor;
		b += factor;
	}

	public double difference(Spectrum spec)
	{
		double[] rgb;
		rgb = spec.getRGB(null);

		double diff = Math.abs(r - rgb[0]) + Math.abs(g - rgb[1]) + Math.abs(b - rgb[2]);

		return diff / 3;
	}

	public double[] getRGB(double[] rgb)
	{
		if(rgb == null)
			rgb = new double[3];
/*
		rgb[0] = Math.min(r, 1.0);
		rgb[1] = Math.min(g, 1.0);
		rgb[2] = Math.min(b, 1.0);
*/
		rgb[0] = r;
		rgb[1] = g;
		rgb[2] = b;

		return rgb;
	}

	public void setRGB(double r, double g, double b)
	{
		this.r = r;
		this.g = g;
		this.b = b;
	}

	public void setRGB(int r, int g, int b)
	{
		this.r = r / 255.;
		this.g = g / 255.;
		this.b = b / 255.;
	}

	public String toString()
	{
		return(getClass().getName() + '@' + Integer.toHexString(hashCode()) + ": " + r + ", " + g + ", " + b);
	}

}
