package pl.webnet.JaTrac;

import java.util.*;

class Surface implements XmlSettable, Indexable
{
	double diffuseCoeff = 0.;
	double specularCoeff = 0.;
	double specularSizeCoeff = 0.;
	double reflectionCoeff = 0.;
	double transmissionCoeff = 0.;
	double refractionIndex = 0.;
	Texture texture = null;
	double weight = 1.;
	static List index = new Vector();
	String name = "unnamed";

	public Surface()
	{
		index.add(this);
	}
	
	public Surface(String name, double d, double s, double ss, double r, double t, double ref, double w)
	{
		index.add(this);

		diffuseCoeff = d;
		specularCoeff = s;
		specularSizeCoeff = 1 / (ss * ss);
		reflectionCoeff = r;
		transmissionCoeff = t;
		refractionIndex = ref;
		weight = w;
	}

	public void setName(String name) { this.name = name; }
	public String getName() { return name; }

	public void setSpecularSize(String size)
	{
		double s = ParamHelper.extractDouble(size);
		specularSizeCoeff = 1 / (s * s);
	}

	public String[] getParamArray()
	{
		String params[] = {
			"name", "name", "",
			"diffuse", "diffuseCoeff", "",
			"specular", "specularCoeff", "",
			"specularsize", "", "setSpecularSize",
			"reflected", "reflectionCoeff", "",
			"transmitted", "transmissionCoeff", "",
			"refraction", "refractionIndex", "",
			"weight", "weight", "",
			"texture", "texture", ""
		};

		return params;
	}
	
	public String[] getAttachArray() { return null; }
	public Object initialize() { return this; }

	static public Surface averageSurfaces(Vector mappings)
	{
		Surface average = new Surface();
		int count = mappings.size();
		double totalWeight = 0;

		if(count != 0)
		{
			for(Iterator i = mappings.iterator(); i.hasNext(); )
			{
				Mapping mn = (Mapping)i.next();
				Surface s = mn.surface;

				average.diffuseCoeff += s.diffuseCoeff * s.weight;
				average.specularCoeff += s.specularCoeff * s.weight;
				average.specularSizeCoeff += s.specularSizeCoeff * s.weight;
				average.reflectionCoeff += s.reflectionCoeff * s.weight;
				average.transmissionCoeff += s.transmissionCoeff * s.weight;
				average.refractionIndex += s.refractionIndex * s.weight;

				totalWeight += s.weight;
			}

			average.diffuseCoeff /= totalWeight;
			average.specularCoeff /= totalWeight;
			average.specularSizeCoeff /= totalWeight;
			average.reflectionCoeff /= totalWeight;
			average.transmissionCoeff /= totalWeight;
			average.refractionIndex /= totalWeight;
		}
		else // return default surface
			average = new Surface("default", 1., 0., 0., 0., 0., 0., 1.);

		return average;
	}
}
