package pl.webnet.JaTrac;

import java.awt.*;

class Triangle extends Primitive implements XmlSettable
{
	static private final int AXIS_X = 1;
	static private final int AXIS_Y = 2;
	static private final int AXIS_Z = 3;

	Point x1, x2, x3;
	Vec E1, E2;
	Vec normal;
	Vec tangent;
	double D;
	double area;
	int dominant;
	static int back = 0;
	static int count = 0;

	public Triangle()
	{
		super();
	}

	public Triangle(Point x1, Point x2, Point x3)
	{
		super();

		this.x1 = new Point(x1);
		this.x2 = new Point(x2);
		this.x3 = new Point(x3);

		initialize();
	}


	public Object initialize()
	{
		Vec edge1 = new Vec(x1.x - x2.x, x1.y - x2.y, x1.z - x2.z);
		Vec edge2 = new Vec(x3.x - x2.x, x3.y - x2.y, x3.z - x2.z);

		normal = edge1.cross(edge2);
		normal.unit();

		tangent = edge1;
		tangent.unit();

		D = - (normal.x * x2.x + normal.y * x2.y + normal.z * x2.z);

		double nx = Math.abs(normal.x);
		double ny = Math.abs(normal.y);
		double nz = Math.abs(normal.z);

		// select plane parallel to one of the axes and map the triangle to it
		if(nx >= ny && nx >= nz)
		{
			E1 = new Vec(x1.y - x3.y, x1.z - x3.z, 0.);
			E2 = new Vec(x2.y - x3.y, x2.z - x3.z, 0.);
			dominant = AXIS_X;
		}
		else
			if(ny >= nx && ny >= nz)
			{
				E1 = new Vec(x1.x - x3.x, x1.z - x3.z, 0.);
				E2 = new Vec(x2.x - x3.x, x2.z - x3.z, 0.);
				dominant = AXIS_Y;
			}
			else
				if(nz >= nx && nz >= ny)
				{
					E1 = new Vec(x1.x - x3.x, x1.y - x3.y, 0.);
					E2 = new Vec(x2.x - x3.x, x2.y - x3.y, 0.);
					dominant = AXIS_Z;
				}
		

		// normalize triangle area to simplify calculations later
		double descale = 1. / -(E1.x * E2.y - E1.y * E2.x);
		E1.scale(descale);
		E2.scale(descale);

		return this;
	}

	public String[] getParamArray()
	{
		final String params[] = {
			"p1", "x1", "",
			"p2", "x2", "",
			"p3", "x3", "",
		};
		
		return ParamHelper.mergeParams(super.getParamArray(), params);
	}

	public String[] getAttachArray() { return null; }

	public int intersect(Ray ray)
	{
		double vd = normal.dot(ray);
		double x = 0, y = 0;
		count++;

		// check if ray is not parallel to the triangle's plane
		if(vd == 0.)
			return 0;

		double v0 = - (normal.dot(ray.origin) + D);
		double t = v0 / vd;

		// check if the intersection is not behind ray's origin
		if(t < Ray.EPSILON)
		{back++;
			return 0;
		}

		// find coordinates of intersection point in 2D triangle's space
		// shifted so that P3 is in (0,0)
		switch(dominant)
		{
			case AXIS_X:
				x = ray.y * t + ray.origin.y - x3.y;
				y = ray.z * t + ray.origin.z - x3.z;
				break;

			case AXIS_Y:
				x = ray.x * t + ray.origin.x - x3.x;
				y = ray.z * t + ray.origin.z - x3.z;
				break;

			case AXIS_Z:
				x = ray.x * t + ray.origin.x - x3.x;
				y = ray.y * t + ray.origin.y - x3.y;
				break;
		}


		double v = E2.x * y - E2.y * x;

		if( v >= 0 && v <= 1.)
		{
			double w = x * E1.y - y * E1.x;

			if(w >= 0 && w <= 1.)
			{
				double u = 1. - v - w;

				if(u >= 0 && u < 1.)
				{
					ray.intersections.add(new Intersection(t, this, Intersection.ENTERING, null));
					return 1;
				}
			}
		}

		return 0;
	}

	public Vec getNormal(Point p)
	{
		return new Vec(normal);		
	}

	public Vec getTangent(Point p, Vec n)
	{
		return new Vec(tangent);
	}
	
	public BoundingBox getBoundingBox(BoundingBox bx)
	{
		bx.set(	(x1.x < x2.x) ? ((x1.x < x3.x) ? x1.x : x3.x) : ((x2.x < x3.x) ? x2.x : x3.x),
						(x1.x > x2.x) ? ((x1.x > x3.x) ? x1.x : x3.x) : ((x2.x > x3.x) ? x2.x : x3.x),
						(x1.y < x2.y) ? ((x1.y < x3.y) ? x1.y : x3.y) : ((x2.y < x3.y) ? x2.y : x3.y),
						(x1.y > x2.y) ? ((x1.y > x3.y) ? x1.y : x3.y) : ((x2.y > x3.y) ? x2.y : x3.y),
						(x1.z < x2.z) ? ((x1.z < x3.z) ? x1.z : x3.z) : ((x2.z < x3.z) ? x2.z : x3.z),
						(x1.z > x2.z) ? ((x1.z > x3.z) ? x1.z : x3.z) : ((x2.z > x3.z) ? x2.z : x3.z) );

		return bx;
	}

	public String toString()
	{
		return(getClass().getName() + '@' + Integer.toHexString(hashCode()) + ": " + x1 + ", " + x2 + ", " + x3 + ". Normal: " + normal + ". D: " + D + ". Dominant: " + dominant);
	}
}
